#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#include "serialbw.h"

#ifdef HAVE_GETOPT_LONG
#include <getopt.h>
#else
/* If not in library, use our own version */
#include "getopt.h"
#endif

static char rcsid[] = "$Id: sermonstats.c,v 0.0 2001/03/18 20:09:32 fox Exp $";

static void
banner(FILE *fp, char *prefix)
{
	fprintf(fp, 
			"%s%s\n"
			"%s\n"
			"%sThis is free software, see the sources for details.\n"
			"%sThis software has NO WARRANTY, use at your OWN RISK\n"
			"%s\n",
			prefix, rcsid,
			prefix, prefix, prefix, prefix);
}

static void 
usage(char *program_name)
{
	banner(stderr, "");
	fprintf(stderr,
			"Usage: %s [OPTIONS] device\n\n"
			"   -h, --help            show a help summary to stderr\n"
			"   -i, --interval        set the minimum interval between process signals\n"
			"   -r, --rate_threshold  set the rate threshold below which processes are signalled\n"
			"   -v, --version         print the serialmon version number on stderr\n"
			"   -z, --zap             zap the protocol and clear down\n"
			, program_name);
}

int
main(int argc, char **argv)
{
	int shmid;
	SERIALBW_PTR ptr;
	key_t devkey;
	char *devname;
	char *program = argv[0];
	double new_rate = 0.0;
	double new_tgap = 0.0;
	int zap = 0;
	/* Process command line options */
	const struct option longopts[] =
	{
		{"help", no_argument, NULL, 'h'},
		{"interval", required_argument, NULL, 'i'},
		{"rate_threshold", required_argument, NULL, 'r'},
		{"version", no_argument, NULL, 'v'},
 		{"zap", no_argument, NULL, 'z'},
		{0, 0, 0, 0},
	};
	int opt;

	while ((opt = getopt_long(argc, argv, "hi:r:vz", longopts, NULL)) != EOF)
	{
		switch (opt)
		{
		case 'h':
			usage(program);
			break;
		case 'r':
			new_rate = atof(optarg);
			break;
		case 'i':
			new_tgap = atof(optarg);
			break;
		case 'v':
			fprintf(stderr, "Version: sermonstats from %s-%s\n", PACKAGE, VERSION);
			exit(0);
			break;
		case 'z':
			zap = 1;
			break;
		case '?':
#if 1
			fprintf(stderr, "unrecognized option.\n");
#endif
			usage(program);
			return 1;
		case ':':
#if 1
			fprintf(stderr, "missing parameter.\n");
#endif
			usage(program);
			return 1;
		default:
			fprintf(stderr,
					"?? getopt returned character code 0x%x ('%c').\n",
					opt, opt);
			return 1;
		}

	}
	argc -= optind - 1;
	argv += optind - 1;

	if((argc > 2)||(argc < 2))
	{
		usage(program);
		return 1;
	}
	devname = argv[1];
	devkey = ftok(devname, 'S');
	if(devkey == (key_t)(-1))
	{
		perror("Failed to get key");
		exit(1);
	}
	/* Make shared segment */
	shmid = shmget(devkey, SEGSIZE, 0);
	if(shmid == -1)
	{
		perror("Failed to get shared memory");
		exit(2);
	}
	ptr = (SERIALBW_PTR)shmat(shmid, 0, 0);
	if(ptr == (SERIALBW_PTR)(-1))
	{
		perror("Failed to map shared memory");
		exit(5);
	}
/*	printf("%d\n", ptr->sbw_flags); */
	if((ptr->sbw_flags & SBW_SERVEROK) == 0)
	{
		printf("NOSERVER\n");
		return 1;
	}
	if(new_rate)
	{
		ptr->new_rate = new_rate;
		while(ptr->new_rate)
			;
	}
	if(new_tgap)
	{
		ptr->new_tgap = new_tgap;
		while(ptr->new_tgap)
			;
	}
	if(zap)
	{
		ptr->rx_req = 0;
		ptr->rx_ack = 0;
		ptr->tx_req = 0;
		ptr->tx_ack = 0;
	}
	printf("%s %8lu %8lu %8lu\n", ptr->sbw_flags & SBW_LINEUP ? "UP" : "DOWN", ptr->rx_count, ptr->tx_count, ptr->txrx_count);
	return 0;
}
