#include <sys/ipc.h>
#include <sys/shm.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include "serialbw.h"

#define DEBUG
#undef DEBUG

static
void sighndl(int signo)
{
	exit(0);
}

int
main(int argc, char **argv)
{
	int shmid;
	SERIALBW_PTR ptr;
	key_t devkey;
	char *devname;
#ifdef DEBUG
	pid_t last_ack = 0, other_req = 0;
#endif
	pid_t mypid = getpid();
	if((argc > 2)||(argc < 2))
	{
		fprintf(stderr, "Usage: %s device\n", argv[0]);
		exit(2);
	}
	devname = argv[1];
	devkey = ftok(devname, 'S');
	if(devkey == (key_t)(-1))
	{
		perror("Failed to get key");
		exit(1);
	}
#ifdef DEBUG
  	printf("key is %d\n", devkey);
#endif
	/* Make shared segment */
	shmid = shmget(devkey, SEGSIZE, 0);
	if(shmid == -1)
	{
		perror("Failed to get shared memory");
		exit(2);
	}
#ifdef DEBUG
  	printf("shmid is %d\n", shmid);
#endif
	ptr = (SERIALBW_PTR)shmat(shmid, 0, 0);
	if(ptr == (SERIALBW_PTR)(-1))
	{
		perror("Failed to map shared memory");
		exit(5);
	}
#ifdef DEBUG
  	printf("sbw_flags == %d\n", ptr->sbw_flags);
#endif
	/* Check server running */
	if((ptr->sbw_flags & SBW_SERVEROK) == 0)
		return 2;
	/* Register ourselves */
	while(ptr->sbw_flags & SBW_LINEUP)
	{
		/* Raise my hand */
		if(ptr->rx_req == 0)
		{
#ifdef DEBUG
  			printf("%d: Registering %d\n", mypid, mypid);
#endif
			ptr->rx_req = mypid;
			continue;
		}
		/* I've been seen - nana-na-nana */
		if(ptr->rx_ack == mypid)
		{
#ifdef DEBUG
			printf("%d: Acked %d\n", mypid, mypid);
#endif
			ptr->rx_req = 0;
			ptr->rx_ack = 0;
			break;
		}
		/* Someone else got there first - Back off a bit */
		if((ptr->rx_req != mypid)||(ptr->rx_ack != mypid))
		{
#ifdef DEBUG
			if(ptr->rx_ack != last_ack)
			{
 				printf("%d: rx_ack == %d\n", mypid, ptr->rx_ack);
				last_ack = ptr->rx_ack;
			}
			if(ptr->rx_req != other_req)
			{
				other_req = ptr->rx_req;
				printf("%d: Other pid = %d\n", mypid, ptr->rx_req);
			}
#endif
			usleep(50000);
		}
	}
	/* Install handler */
	while((ptr->sbw_flags & SBW_LINEUP)&&(ptr->rx_go != mypid))
	{
		signal(SIGCONT,sighndl);
		/* Now wait for signal */
		sleep(600);
	}
	if(ptr->sbw_flags & SBW_LINEUP)
		return 0;
#ifdef DEBUG
  	printf("Line dropped\n");
#endif
	return 1;
}
